<?php
namespace Notifal;
use mysqli_result;
use stdClass;

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

/**
 * Notifal db
 *
 * Notifal db handler class is responsible for setting up database connection
 *
 * @since 1.0.0
 */
require_once(ABSPATH . 'wp-admin/includes/upgrade.php');

class db {

    /**
     * return table prefix
     *
     * @since 1.0.0
     * @return string
     */
    public static function get_prefix(): string
    {
        global $wpdb;
        $prefix = $wpdb->prefix;
        return $prefix;
    }

    /**
     * add prefix to table name
     *
     * @since 1.0.0
     * @param $table_name
     * @return string
     */
    public static function add_prefix_table($table_name): string
    {
        $table = self::get_prefix() . $table_name;
        return $table;
    }

    /**
     * Creat table
     *
     * @since 1.0.0
     * @param string $table_name
     * @param array $columns
     * @param string $primary_key
     * @param string $index_key
     * @return void
     */
    public static function create_table(string $table_name,array $columns,string $primary_key , string $index_key=''){
        global $wpdb;
        $charset_collate = $wpdb->get_charset_collate();
        $table = self::add_prefix_table($table_name);
        $column_query='';
        $i=0;
        foreach ($columns as $column){
            $column_query .= $column['name'].' '. $column['type']. ' ' . $column['extra_data'] . ',';
            $i++;
            if($i === sizeof($columns)){
                $column_query .= "PRIMARY KEY  ($primary_key)";
                if(!empty($index_key)){
                    $column_query .= ",".$index_key;
                }
            }
        }

        $sql = "CREATE TABLE " . $table . " ($column_query)$charset_collate;";
        dbDelta($sql);
    }

    /**
     * insert to table
     * return true if inserted successfully otherwise return false
     *
     * @since 1.0.0
     * @param $data
     * @return int|false
     */
    public static function insert_in_table($table_name,$data,$format=null)
    {
        if(maintenance::is_features_inactive()){
            return false;
        }
        global $wpdb;
        $table= self::add_prefix_table($table_name);
        $insert=$wpdb->insert(
            $table,
            $data,
            $format
        );

        if($insert){
            return $wpdb->insert_id;
        }
        return false;
    }

    /**
     * return true is table updated successfully otherwise false
     *
     * @since 1.0.0
     * @param string $table_name
     * @param array $data
     * @param array $condition
     * @param $where_format
     * @return bool
     */
    public static function update_table(string $table_name,array $data, array $condition,$where_format=null,$data_format=null): bool
    {
        if(maintenance::is_features_inactive()){
            return false;
        }
        global $wpdb;
        $table = self::add_prefix_table($table_name);

        $update=$wpdb->update(
            $table,
            $data,
            $condition,
            $data_format,
            $where_format
        );
        if($update){
            return true;
        }
        return false;
    }


    /**
     * Execute $query
     *
     * @since 1.0.0
     * @param string $query
     * @return void
     * @since
     */
    public static function run_query(string $query){
        global $wpdb;
        $wpdb->query(
            $query
        );
    }

    /**
     * returns row where $column = $value
     *
     * @since 1.0.0
     * @param string $table_name
     * @param string $column
     * @param string $value
     * @return array|object|stdClass|null
     */
    public static function get_row_by_column(string $table_name, string $column,string $value){
        global $wpdb;
        $table = self::add_prefix_table($table_name);
        return $wpdb->get_row("SELECT * FROM $table WHERE $column='$value' ");

    }


    /**
     * return latest id before inserted to table
     *
     * @since 1.0.0
     * @param string $table_name
     * @return false|mixed
     */
    public static function get_latest_id(string $table_name){
        if(!self::if_table_exist($table_name)){
            return false;
        }
        global $wpdb;
        $table = self::add_prefix_table($table_name);
        $db_name = DB_NAME;
        $query = "SELECT AUTO_INCREMENT FROM  INFORMATION_SCHEMA.TABLES WHERE TABLE_SCHEMA = '$db_name' AND TABLE_NAME='$table' ";
        return $wpdb->get_row($query,ARRAY_A)['AUTO_INCREMENT'];
    }


    /**
     * return true if table exist is db otherwise return false
     *
     * @since 1.0.0
     * @param $table_name
     * @return bool
     */
    public static function if_table_exist($table_name): bool
    {
        global $wpdb;
        $table = self::add_prefix_table($table_name);
        $db_name = DB_NAME;
        $query = "SELECT AUTO_INCREMENT FROM  INFORMATION_SCHEMA.TABLES WHERE TABLE_SCHEMA = '$db_name' AND TABLE_NAME='$table' ";
        $row= $wpdb->get_row($query,ARRAY_A);
        if(empty($row)) return false;
        return true;

    }


    /**
     * get table rows
     *
     * @param string $table_name
     * @param string $column
     * @param array $conditions
     * @param string $order_by
     * @param int $limit
     * @param int $offset
     * @param string $operator
     * @return array|object|stdClass|null
     * @since 1.0.0
     */
    public static function get_table_rows(string $table_name,string $column='all',array $conditions=[],string $order_by='id', int $limit=-1,int $offset=0,string $operator='AND'){
        if(maintenance::is_features_inactive()){
            return [];
        }
        if(!self::if_table_exist($table_name)) return [];
        global $wpdb;
        $table = self::add_prefix_table($table_name);
        $query = 'SELECT ';
        if($column=='all'){
            $query .='*';
        }else{
            $query.= $column;
        }

        $query.=' From '. $table ;

        if(!empty($conditions)){
            $i=0;
            $conditions_query='';
            foreach ($conditions as $condition){
                if($i==0)
                $conditions_query .= $condition;
                else
                $conditions_query .= ' '.$operator .' ' . $condition;
                $i++;
            }
            $query .= ' Where ' . $conditions_query;
        }
        if($limit==-1){
            $query.=' ORDER BY ' .$order_by. ' DESC ';
        }else{
            $query.=' ORDER BY ' .$order_by. ' DESC LIMIT ' . $limit ;
        }
        if($offset!=0){
            $query.= ' OFFSET '. $offset;
        }

        $results = $wpdb->get_results($query);

        return $results;

    }


    /**
     * remove row when $condition is found
     *
     * @param string $table_name
     * @param array $condition
     * @param string[]|string $where_format
     * @return bool|int|mysqli_result|string
     * @since 1.1.0
     */
    public static function remove_row(string $table_name,array $condition,$where_format= null){
        if(maintenance::is_features_inactive()){
            return false;
        }
        if(!self::if_table_exist($table_name)) return '';
        global $wpdb;
        $table = self::add_prefix_table($table_name);

        $result= $wpdb->delete( $table, $condition ,$where_format );
        if($result){
            return $result;
        }
        return false;
    }

    /**
     * return orders by products
     *
     * @param $product_type
     * @param $products_id
     * @param $order_threshold_num
     * @param $order_status
     * @return array|mixed|stdClass|\WC_Order[]|null
     * @since 1.1.0
     */
    public static function get_orders_by_products($product_type,$products_id,$order_threshold_num,$order_status=[]){
        if(maintenance::is_features_inactive()){
            return [];
        }
        if(empty($order_status)){
            $order_status = ['wc-processing','wc-completed'];
        }
        if($product_type!=='all-products'){
            $orders = [];
            $orders=apply_filters('Notifal_get_orders_by_products',$orders,$products_id,$order_status,$product_type,$order_threshold_num);

        }else{
            $time_in_seconds = $order_threshold_num*DAY_IN_SECONDS;
            $orders = wc_get_orders( array(
                'limit' => -1,
                'return' => 'ids',
                'status' => $order_status,
                'date_created' => '>' . ( time() - $time_in_seconds ),
            ) );
        }

        return $orders;

    }

    /**
     * return id is $key related to $id is exited otherwise return false
     *
     * @param string $table_name
     * @param $column
     * @param $where
     * @return string|null
     * @since 1.1.0
     */
    public static function get_key_value(string $table_name,$column,$where): ?string
    {
        global $wpdb;
        $table = self::add_prefix_table($table_name);
        $existing_meta=$wpdb->get_var($wpdb->prepare(
            "SELECT $column FROM $table WHERE $where"
        ));
        return $existing_meta;
    }

    /**
     * Check if a row exists in the specified table based on provided conditions.
     *
     * @param string $table_name Name of the table (without prefix).
     * @param array $columns Associative array of column => value pairs for the WHERE clause.
     * @param array|null $select_columns Optional. Array of columns to select. Defaults to ['id'].
     * @return object|null Returns the row object if found, or null if no row exists.
     * @since 1.1.0
     */
    public static function get_row_by_conditions(string $table_name, array $columns, ?array $select_columns = ['id']): ?object
    {
        global $wpdb;

        $conditions = [];
        $values = [];
        foreach ($columns as $column => $value) {
            $conditions[] = "{$column} = %s";
            $values[] = $value;
        }
        $where_clause = implode(' AND ', $conditions);

        $select_columns_string = implode(', ', $select_columns);
        $query = $wpdb->prepare(
            "SELECT {$select_columns_string} FROM {$wpdb->prefix}{$table_name} WHERE {$where_clause} LIMIT 1",
            ...$values
        );

        return $wpdb->get_row($query);
    }



    public static function get_all_rows($table_name){
        global $wpdb;
        $table_name = self::add_prefix_table($table_name);
        return $wpdb->get_results("SELECT * FROM {$table_name}");
    }

    /**
     * Retrieves a specific column from a database table with optional conditions.
     *
     * @since 1.1.0
     *
     * @param string $table_name Name of the table to query.
     * @param string $column Name of the column to retrieve.
     * @param array  $conditions Optional conditions for the query, each in the format ['column_name' => value].
     * @param string $order_by Column name to order results by (default: 'id').
     * @param int    $limit Maximum number of rows to retrieve (default: -1 for no limit).
     * @param int    $offset Number of rows to skip before starting to retrieve (default: 0).
     * @param string $operator Logical operator for conditions ('AND' or 'OR', default: 'AND').
     *
     * @return array List of column values matching the query.
     */
    public static function get_column_data(
        string $table_name,
        string $column = 'id',
        array $conditions = [],
        string $order_by = 'id',
        int $limit = -1,
        int $offset = 0,
        string $operator = 'AND'
    ): array {
        if (maintenance::is_features_inactive()) {
            return [];
        }
        if (!self::if_table_exist($table_name)) {
            return [];
        }
        global $wpdb;
        $table = self::add_prefix_table($table_name);
        $query = "SELECT {$column} FROM {$table}";
        $query_conditions = [];
        $query_params = [];
        if (!empty($conditions)) {
            foreach ($conditions as $col => $value) {
                $query_conditions[] = "{$col} = %s";
                $query_params[] = $value;
            }
            $query .= " WHERE " . implode(" {$operator} ", $query_conditions);
        }
        $query .= " ORDER BY {$order_by} DESC";
        if ($limit > -1) {
            $query .= " LIMIT %d";
            $query_params[] = $limit;
        }
        if ($offset > 0) {
            $query .= " OFFSET %d";
            $query_params[] = $offset;
        }
        if (!empty($query_params)) {
            $query = $wpdb->prepare($query, ...$query_params);
        }

        $results = $wpdb->get_col($query);

        return $results;
    }




    /**
     * Retrieves distinct rows from a table based on specified columns and conditions.
     *
     * @param string $table_name     The name of the table to query.
     * @param array  $columns        The columns to retrieve distinct rows for.
     * @param array  $conditions     Optional. An associative array of conditions to filter the rows. Default is an empty array.
     * @return array|null            An array of distinct rows or null on failure.
     */
    public static function get_distinct_rows(string $table_name, array $columns, array $conditions = []): ?array {
        global $wpdb;
        $table_name = db::add_prefix_table($table_name);

        $columns_string = implode(', ', $columns);
        $where_clause = '';
        $values = [];

        if (!empty($conditions)) {
            $conditions_parts = [];
            foreach ($conditions as $column => $value) {
                $conditions_parts[] = "{$column} = %s";
                $values[] = $value;
            }
            $where_clause = 'WHERE ' . implode(' AND ', $conditions_parts);
        }

        $query = "SELECT DISTINCT {$columns_string} FROM {$table_name} {$where_clause}";

        if (!empty($values)) {
            $query = $wpdb->prepare($query, ...$values);
        }

        return $wpdb->get_results($query);
    }


    /**
     * Retrieves a single value from a specified table based on given conditions.
     *
     * This function constructs a SQL query to select a specific column's value
     * from a table, applying optional conditions. It utilizes WordPress's
     * $wpdb->prepare() method to ensure safe query execution.
     *
     * @param string $table_name   The name of the table (without the $wpdb->prefix).
     * @param string $column       The column name to retrieve the value from.
     * @param array $conditions   Optional. An associative array of conditions where keys are column names
     *                             and values are the values to match. Default empty array.
     * @param array $placeholders Optional. An array of placeholders corresponding to the conditions.
     *                             Default empty array.
     * @return mixed The value of the specified column, or null if not found.
     *@since 1.1.0
     *
     */
//    public static function get_var_from_table(string $table_name, string $column, array $conditions = [], array $placeholders = [])
//    {
//        global $wpdb;
//
//        $table_name = self::add_prefix_table($table_name);
//
//
//        $query = "SELECT $column FROM $table_name";
//        $condition_strings = [];
//
//        foreach ($conditions as $col => $value) {
//            $condition_strings[] = "$col = %s";
//            $placeholders[] = $value;
//        }
//
//        if (!empty($condition_strings)) {
//            $query .= ' WHERE ' . implode(' AND ', $condition_strings);
//        }
//
//        if (!empty($placeholders)) {
//            $query = $wpdb->prepare($query, $placeholders);
//        }
//
//        $result = $wpdb->get_var($query);
//
//        if (is_null($result)) {
//            return 0;
//        }
//
//        return $result;
//    }

    /**
     * Retrieves a single value from a specified table based on given conditions.
     *
     * This function constructs a SQL query to select a specific column's value
     * from a table, applying optional conditions.
     *
     * @param string $table_name   The name of the table (without the $wpdb->prefix).
     * @param string $column       The column name to retrieve the value from.
     * @param array  $conditions   Optional. An associative array of conditions where keys are column names
     *                             and values are the values to match. Default empty array.
     * @param array  $placeholders Optional. An array of placeholders corresponding to the conditions.
     *                             Default empty array.
     * @return mixed The value of the specified column, or null if not found.
     * @since 1.1.0
     */
    public static function get_var_from_table(string $table_name, string $column, array $conditions = [], array $placeholders = [])
    {
        global $wpdb;

        $table_name = self::add_prefix_table($table_name);

        // Start building the query
        $query = "SELECT $column FROM $table_name";
        $condition_strings = [];

        // Iterate over the conditions
        foreach ($conditions as $col => $value) {
            // If the value is an array, use the 'IN' clause
            if (is_array($value)) {
                $in_placeholders = implode(',', array_fill(0, count($value), '%s'));
                $condition_strings[] = "$col IN ($in_placeholders)";
                // Merge the array of values into the placeholders
                $placeholders = array_merge($placeholders, $value);
            } else {
                // Otherwise, handle as a simple equality condition
                $condition_strings[] = "$col = %s";
                $placeholders[] = $value;
            }
        }

        // If there are conditions, append them to the query
        if (!empty($condition_strings)) {
            $query .= ' WHERE ' . implode(' AND ', $condition_strings);
        }

        // Prepare and execute the query
        if (!empty($placeholders)) {
            $query = $wpdb->prepare($query, ...$placeholders);
        }

        // Get the result
        $result = $wpdb->get_var($query);

        return $result ?: 0;
    }



}