<?php
namespace Notifal;

if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly.
}

/**
 * Notifal maintenance.
 *
 * Notifal maintenance handler class is responsible for setting up Notifal
 * activation and uninstallation hooks.
 *
 * @since 1.0.0
 */

require_once( ABSPATH . 'wp-admin/includes/plugin.php' );
class maintenance {


    public function __construct()
    {
        add_action('admin_notices', [$this,'activation_admin_notices'] );
        add_action('admin_notices', [$this,'malfunction_admin_notices'] );
        add_action('admin_init',[$this,'check_plugin_prerequisites']);
    }


    /**
     * Print errors in any
     *
     * @since 1.0.0
     * @return void
     */
    public function activation_admin_notices(){
        global $wp_version;
        if(!self::is_prerequisites_checked()){
            if ( ! version_compare( PHP_VERSION, '7.4', '>=' )) {
                echo '<div class="error"><p>'.esc_html__('Activation failed! PHP is outdated.',Notifal_DOMAIN).'</p></div>';
            }
            if($wp_version < 5.6){
                echo '<div class="error"><p>'.esc_html__('Activation failed! WordPress need to be at least 5.6',Notifal_DOMAIN).'</p></div>';
            }
        }




    }

    /**
     * Execute on Notifal activation
     *
     * @since 1.0.0
     * @return void
     */
    public static function activation(): void
    {
        notif::create_notification_table();
        notif::create_notification_meta_table();
        notif::insert_precreated_notifs();
    }


    /**
     * Execute on Notifal deactivation
     *
     * @since 1.0.0
     */
    public static function deactivation(): void
    {
        if(class_exists('Notifal_Pro')){
            deactivate_plugins( Notifal_Pro_BASE);
        }
    }

    /**
     * Execute on admin init to check if plugin requirement or pro is installed
     *
     * @since 1.0.0
     * @return void
     */
    public function check_plugin_prerequisites(): void
    {
        if(!self::is_prerequisites_checked()){
            deactivate_plugins( Notifal_BASE);
            unset( $_GET['activate'] );
        }
        if(!self::is_pro_active()){
            if(sizeof(notif::get_notifs_by_status('Active'))>1){
               global $wpdb;
               $notif_table = $wpdb->prefix.notif::$notifal_table;
               $query= "UPDATE $notif_table SET `status` = 'Inactive'";
               db::run_query($query);
            }
        }
    }


    /**
     * Check if features is active
     *
     * @since 1.0.0
     * @return bool
     */
    public static function is_features_inactive(): bool
    {
        if(!class_exists(Notifal_NAMESPACE.'\\features')){
            return true;
        }
        return false;
    }



    /**
     * Initialize Notifal Maintenance.
     *
     * @return void
     * @since 1.0.0
     */
    public static function init(): void
    {
        if(self::is_prerequisites_checked()){
            register_activation_hook(Notifal_BASE, [ __CLASS__, 'activation' ] );
            register_deactivation_hook(Notifal_BASE, [ __CLASS__, 'deactivation' ] );
        }
    }

    /**
     * return true if pro installed otherwise return false
     *
     * @since 1.0.0
     * @return bool
     */
    public static function is_pro_installed(): bool
    {
        if(class_exists('Notifal_Pro')){
            return true;
        }
        return false;
    }

    /**
     * return true if pro is active otherwise return false
     *
     * @since 1.0.0
     * @return bool
     */

    public static function is_pro_active(){
        if(class_exists('Notifal_Pro') && !empty(get_option('Notifal_Pro_license'))){
            return true;
        }
        return false;
    }

    /**
     * return true if pro is installed but not connected otherwise return false
     *
     * @since 1.0.0
     * @return bool
     */
    public static function is_pro_active_but_not_connected(){
        if(class_exists('Notifal_Pro') && empty(get_option('Notifal_Pro_license'))){
            return true;
        }
        return false;
    }


    /**
     * return true if woocommerce is active otherwise return false
     *
     *
     * @since 1.0.0
     * @return bool
     */
    public static function is_woocommerce_active(){
        if(class_exists('WooCommerce')) return true;
        return false;
    }

    /**
     * returns true if Notifal requirement is met otherwise returns false
     *
     * @since  1.0.0
     * @return bool
     */

    public static function is_prerequisites_checked(): bool
    {
        global $wp_version;
        if(version_compare( PHP_VERSION, '7.4', '>=') && $wp_version > 5.6){
            return true;
        }
        return false;
    }

    /**
     * Print notice if the plugin is malfunctioned.
     *
     * @since 1.0.0
     * @return void
     */
    public function malfunction_admin_notices()
    {
        if (self::is_features_inactive()) {
            echo '<div class="error"><p>' . esc_html__('Some files are missing, causing Notifal to stop working. Please contact us for assistance in resolving this issue.', Notifal_DOMAIN) . '<a href="' . Notifal_WEBSITE . '" target="_blank">' . esc_html__('Contact Us', Notifal_DOMAIN) . '</a></p></div>';
        }
    }




}
